**Free
Ctl-Opt NoMain DecEdit(*JobRun);
//
// NumPre
// ------
// Functions to represent numbers expressed in a short form.
//
// E.g.: 10000:   10 T
//       1024:     1 KB
//
// Jesper Wachs, Version 1, April 2019.
//
// The source member are delivered 'as is'.
//
// Neither Jesper Wachs nor anyone else who has been involved in
// the creation, production or delivery of this product shall be
// liable for any direct, indirect, consequential or incidental
// damages (including damages for loss of business profits,
// business interruption, loss of business information, and the
// like) arising out of the use or inability to use such product
// even if Jesper Wachs has been advised of the possibility of
// such damages.
//

// Copymember for exported functions.
// ----------------------------------
/copy qrpglesrc,numpre_pr

// Global work fields.
// -------------------

// Default suffix for 1024 based numbers.
Dcl-S SuffixesByte Char(50) Inz('B         KB        +
                                 MB        GB        TB        ');

// Default suffix for stand 1000 based numbers.
Dcl-S SuffixesStd Char(50) Inz('          T         +
                                Mil       Mia       Bil       ');

// Global constants.
// -----------------

Dcl-C CEDITCODEP Const('P');

// Exported funtions.
// ------------------
//
// Nbr2BPre
// ---------
// Byte representation (1024 based)

Dcl-Proc Nbr2BPre Export;
Dcl-PI Nbr2BPre VarChar(19);
   iNumber Int(20:0) Const;
   iDecimal Ind Value;
End-PI;

Dcl-C LIMITBYTEK Const(1024);
Dcl-C LIMITBYTEM Const(1048576);
Dcl-C LIMITBYTEG Const(1073741824);
Dcl-C LIMITBYTET Const(1099511627776);

Dcl-S CalcValue Packed(11:1);
Dcl-S IntValue Packed(9:0);
Dcl-S Offset Packed(3:0);
Dcl-S oResult VarChar(20);
Dcl-S wNumber Like(iNumber);

// Handle negative numbers. Done to have comparisons work as expected.

If iNumber < *Zeros;
   wNumber = iNumber * -1;
Else;
   wNumber = iNumber;
EndIf;

Select;

   When wNumber < LimitByteK;
      Offset = 1;
      CalcValue = iNumber;
      // Force to no decimal as this is the 1's.
      iDecimal = *Off;

   When wNumber < LimitByteM;
      Offset = 11;
      Eval(H) CalcValue = iNumber / LimitByteK;

   When wNumber < LimitByteG;
      Offset = 21;
      Eval(H) CalcValue = iNumber / LimitByteM;

   When wNumber < LimitByteT;
      Offset = 31;
      Eval(H) CalcValue = iNumber / LimitByteG;

   Other;
      Offset = 41;
      Eval(H) CalcValue = iNumber / LimitByteT;
EndSl;

// Format with or without one decimal.

If iDecimal = *On;
   oResult = %TrimL(%EditC(CalcValue: cEditCodeP));
Else;
   Eval(H) IntValue = CalcValue;
   oResult = %TrimL(%EditC(IntValue: cEditCodeP));
EndIf;
oResult = oResult + ' ' + %SubSt(SuffixesByte: Offset: 10);

Return oResult;

End-Proc Nbr2BPre;
//
// Num2BSuff
// ---------
// Set the suffixes for 1024-based calculation routine.

Dcl-Proc Num2BSuff Export;
Dcl-PI Num2BSuff;
    iMask Char(50);
End-PI;

SuffixesByte = iMask;

Return;

End-Proc Num2BSuff;
//
// Nbr2SPre
// --------
// Standard representation (1000 based).

Dcl-Proc Nbr2SPre Export;
Dcl-PI Nbr2SPre VarChar(19);
   iNumber Int(20:0) Const;
   iDecimal Ind Value;
End-PI;

Dcl-C LIMITSTDT Const(1000);
Dcl-C LIMITSTDM Const(1000000);
Dcl-C LIMITSTDMI Const(1000000000);
Dcl-C LIMITSTDB Const(1000000000000);

Dcl-S CalcValue Packed(11:1);
Dcl-S IntValue Packed(9:0);
Dcl-S Offset Packed(3:0);
Dcl-S oResult VarChar(20);
Dcl-S wNumber Like(iNumber);

// Handle negative numbers. Done to have comparisons work as expected.

If iNumber < *Zeros;
   wNumber = iNumber * -1;
Else;
   wNumber = iNumber;
EndIf;

Select;

   When wNumber < LimitStdT;
      Offset = 1;
      CalcValue = iNumber;

      // Force to no decimal as this is the 1's.
      iDecimal = *Off;

   When wNumber < LimitStdM;
      Offset = 11;
      Eval(H) CalcValue = iNumber / LimitStdT;

   When wNumber < LimitStdMi;
      Offset = 21;
      Eval(H) CalcValue = iNumber / LimitStdM;

   When wNumber < LimitStdB;
      Offset = 31;
      Eval(H) CalcValue = iNumber / LimitStdMi;

   Other;
      Offset = 41;
      Eval(H) CalcValue = iNumber / LimitStdB;
EndSl;

// Format with or without one decimal.

If iDecimal = *On;
   oResult = %TrimL(%EditC(CalcValue: cEditCodeP));
Else;
   Eval(H) IntValue = CalcValue;
   oResult = %TrimL(%EditC(IntValue: cEditCodeP));
EndIf;
oResult = oResult + ' ' + %SubSt(SuffixesStd: Offset: 10);

Return oResult;

End-Proc Nbr2SPre;
//
// Num2SSuff
// ---------
// Set the suffixes for 1000-based calculation routine.

Dcl-Proc Num2SSuff Export;
Dcl-PI Num2SSuff;
   iMask Char(50);
End-PI;

SuffixesStd = iMask;

Return;

End-Proc Num2SSuff;
